# Copyright (c) DP Technology.
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.
from IPython import embed as debug_embedded
import logging
import os
from collections.abc import Iterable
from sklearn.metrics import roc_auc_score
from xmlrpc.client import Boolean

import numpy as np
import torch
from tqdm import tqdm
from unicore import checkpoint_utils
import unicore
from unicore.data import (AppendTokenDataset, Dictionary, EpochShuffleDataset,
                          FromNumpyDataset, NestedDictionaryDataset,
                          PrependTokenDataset, RawArrayDataset,LMDBDataset, RawLabelDataset,
                          RightPadDataset, RightPadDataset2D, TokenizeDataset,SortDataset,data_utils)
from unicore.tasks import UnicoreTask, register_task
from unimol.data import (AffinityDataset, CroppingPocketDockingPoseDataset,CroppingPocketDockingPoseTestDataset,
                         CrossDistanceDataset, DistanceDataset,
                         EdgeTypeDataset, KeyDataset, LengthDataset,
                         NormalizeDataset, NormalizeDockingPoseDataset,
                         PrependAndAppend2DDataset, RemoveHydrogenDataset,
                         RemoveHydrogenPocketDataset, RightPadDatasetCoord,
                         RightPadDatasetCross2D, TTADockingPoseDataset, AffinityTestDataset, AffinityValidDataset, AffinityMolDataset, AffinityPocketDataset, ResamplingDataset,
                         AffinityPocketMatchingDataset)
#from skchem.metrics import bedroc_score
from rdkit.ML.Scoring.Scoring import CalcBEDROC, CalcAUC, CalcEnrichment
from sklearn.metrics import roc_curve


from Bio.SVDSuperimposer import SVDSuperimposer

logger = logging.getLogger(__name__)


def re_new(y_true, y_score, ratio):
    fp = 0
    tp = 0
    p = sum(y_true)
    n = len(y_true) - p
    num = ratio*n
    sort_index = np.argsort(y_score)[::-1]
    for i in range(len(sort_index)):
        index = sort_index[i]
        if y_true[index] == 1:
            tp += 1
        else:
            fp += 1
            if fp>= num:
                break
    return (tp*n)/(p*fp)


def calc_re(y_true, y_score, ratio_list):
    fpr, tpr, thresholds = roc_curve(y_true, y_score, pos_label=1)
    #print(fpr, tpr)
    res = {}
    res2 = {}
    total_active_compounds = sum(y_true)
    total_compounds = len(y_true)

    # for ratio in ratio_list:
    #     for i, t in enumerate(fpr):
    #         if t > ratio:
    #             #print(fpr[i], tpr[i])
    #             if fpr[i-1]==0:
    #                 res[str(ratio)]=tpr[i]/fpr[i]
    #             else:
    #                 res[str(ratio)]=tpr[i-1]/fpr[i-1]
    #             break
    
    for ratio in ratio_list:
        res2[str(ratio)] = re_new(y_true, y_score, ratio)

    #print(res)
    #print(res2)
    return res2

def cal_metrics(y_true, y_score, alpha):
    """
    Calculate BEDROC score.

    Parameters:
    - y_true: true binary labels (0 or 1)
    - y_score: predicted scores or probabilities
    - alpha: parameter controlling the degree of early retrieval emphasis

    Returns:
    - BEDROC score
    """
    
        # concate res_single and labels
    scores = np.expand_dims(y_score, axis=1)
    y_true = np.expand_dims(y_true, axis=1)
    scores = np.concatenate((scores, y_true), axis=1)
    # inverse sort scores based on first column
    scores = scores[scores[:,0].argsort()[::-1]]
    bedroc = CalcBEDROC(scores, 1, 80.5)
    count = 0
    # sort y_score, return index
    index  = np.argsort(y_score)[::-1]
    for i in range(int(len(index)*0.005)):
        if y_true[index[i]] == 1:
            count += 1
    auc = CalcAUC(scores, 1)
    ef_list = CalcEnrichment(scores, 1, [0.005, 0.01, 0.02, 0.05])
    ef = {
        "0.005": ef_list[0],
        "0.01": ef_list[1],
        "0.02": ef_list[2],
        "0.05": ef_list[3]
    }
    re_list = calc_re(y_true, y_score, [0.005, 0.01, 0.02, 0.05])
    return auc, bedroc, ef, re_list



@register_task("pocket_matching")
class PocketMatching(UnicoreTask):
    """Task for training transformer auto-encoder models."""

    @staticmethod
    def add_args(parser):
        """Add task-specific arguments to the parser."""
        parser.add_argument(
            "data",
            help="downstream data path",
        )
        parser.add_argument(
            "--finetune-mol-model",
            default=None,
            type=str,
            help="pretrained molecular model path",
        )
        parser.add_argument(
            "--finetune-pocket-model",
            default=None,
            type=str,
            help="pretrained pocket model path",
        )
        parser.add_argument(
            "--dist-threshold",
            type=float,
            default=6.0,
            help="threshold for the distance between the molecule and the pocket",
        )
        parser.add_argument(
            "--max-pocket-atoms",
            type=int,
            default=256,
            help="selected maximum number of atoms in a pocket",
        )
        parser.add_argument(
            "--test-model",
            default=False,
            type=Boolean,
            help="whether test model",
        )
        parser.add_argument("--reg", action="store_true", help="regression task")

    def __init__(self, args, pocket_dictionary):
        super().__init__(args)
        self.pocket_dictionary = pocket_dictionary
        self.seed = args.seed
        self.pocket_mask_idx = pocket_dictionary.add_symbol("[MASK]", is_special=True)


    @classmethod
    def setup_task(cls, args, **kwargs):
        pocket_dictionary = Dictionary.load(os.path.join(args.data, "dict_pkt.txt"))
        logger.info("pocket dictionary: {} types".format(len(pocket_dictionary)))
        return cls(args, pocket_dictionary)

    def load_dataset(self, split, **kwargs):
        """Load a given dataset split.
        'smi','pocket','atoms','coordinates','pocket_atoms','pocket_coordinates','holo_coordinates','holo_pocket_coordinates','scaffold'
        Args:
            split (str): name of the data scoure (e.g., bppp)
        """
        #print(1,split)
        data_path = os.path.join(self.args.data, split + ".lmdb")
        dataset = LMDBDataset(data_path)

            
        dataset = AffinityPocketMatchingDataset(
            dataset,
            self.args.seed,
            "atoms_a",
            "coordinates_a",
            "atoms_b",
            "coordinates_b",
            "label"
        )
        tgt_dataset = KeyDataset(dataset, "label")
            


        def PrependAndAppend(dataset, pre_token, app_token):
            dataset = PrependTokenDataset(dataset, pre_token)
            return AppendTokenDataset(dataset, app_token)

        dataset = RemoveHydrogenPocketDataset(
            dataset,
            "atoms_a",
            "coordinates_a",
            "holo_pocket_coordinates_a",
            True,
            True,
        )
        dataset = CroppingPocketDockingPoseDataset(
            dataset,
            self.seed,
            "atoms_a",
            "coordinates_a",
            "holo_pocket_coordinates_a",
            self.args.max_pocket_atoms,
        )

        dataset = RemoveHydrogenPocketDataset(
            dataset,
            "atoms_b",
            "coordinates_b",
            "holo_pocket_coordinates_b",
            True,
            True,
        )
        dataset = CroppingPocketDockingPoseDataset(
            dataset,
            self.seed,
            "atoms_b",
            "coordinates_b",
            "holo_pocket_coordinates_b",
            self.args.max_pocket_atoms,
        )




        apo_dataset = NormalizeDataset(dataset, "coordinates_a")
        apo_dataset = NormalizeDataset(apo_dataset, "coordinates_b")



        src_pocket_dataset_a = KeyDataset(apo_dataset, "atoms_a")
        src_pocket_dataset_a = TokenizeDataset(
            src_pocket_dataset_a,
            self.pocket_dictionary,
            max_seq_len=self.args.max_seq_len,
        )
        coord_pocket_dataset_a = KeyDataset(apo_dataset, "coordinates_a")
        src_pocket_dataset_a = PrependAndAppend(
            src_pocket_dataset_a,
            self.pocket_dictionary.bos(),
            self.pocket_dictionary.eos(),
        )
        pocket_edge_type_a = EdgeTypeDataset(
            src_pocket_dataset_a, len(self.pocket_dictionary)
        )
        coord_pocket_dataset_a = FromNumpyDataset(coord_pocket_dataset_a)
        distance_pocket_dataset_a = DistanceDataset(coord_pocket_dataset_a)
        coord_pocket_dataset_a = PrependAndAppend(coord_pocket_dataset_a, 0.0, 0.0)
        distance_pocket_dataset_a = PrependAndAppend2DDataset(
            distance_pocket_dataset_a, 0.0
        )

        src_pocket_dataset_b = KeyDataset(apo_dataset, "atoms_b")
        src_pocket_dataset_b = TokenizeDataset(
            src_pocket_dataset_b,
            self.pocket_dictionary,
            max_seq_len=self.args.max_seq_len,
        )
        coord_pocket_dataset_b = KeyDataset(apo_dataset, "coordinates_b")
        src_pocket_dataset_b = PrependAndAppend(
            src_pocket_dataset_b,
            self.pocket_dictionary.bos(),
            self.pocket_dictionary.eos(),
        )
        pocket_edge_type_b = EdgeTypeDataset(
            src_pocket_dataset_b, len(self.pocket_dictionary)
        )
        coord_pocket_dataset_b = FromNumpyDataset(coord_pocket_dataset_b)
        distance_pocket_dataset_b = DistanceDataset(coord_pocket_dataset_b)
        coord_pocket_dataset_b = PrependAndAppend(coord_pocket_dataset_b, 0.0, 0.0)
        distance_pocket_dataset_b = PrependAndAppend2DDataset(
            distance_pocket_dataset_b, 0.0
        )

        nest_dataset = NestedDictionaryDataset(
            {
                "net_input": {
                    "pocket_src_tokens_a": RightPadDataset(
                        src_pocket_dataset_a,
                        pad_idx=self.pocket_dictionary.pad(),
                    ),
                    "pocket_src_distance_a": RightPadDataset2D(
                        distance_pocket_dataset_a,
                        pad_idx=0,
                    ),
                    "pocket_src_edge_type_a": RightPadDataset2D(
                        pocket_edge_type_a,
                        pad_idx=0,
                    ),
                    "pocket_src_tokens_b": RightPadDataset(
                        src_pocket_dataset_b,
                        pad_idx=self.pocket_dictionary.pad(),
                    ),
                    "pocket_src_distance_b": RightPadDataset2D(
                        distance_pocket_dataset_b,
                        pad_idx=0,
                    ),
                    "pocket_src_edge_type_b": RightPadDataset2D(
                        pocket_edge_type_b,
                        pad_idx=0,
                    ),

                },
                "target": {
                    "finetune_target": RawLabelDataset(tgt_dataset),
                },
            },
        )
        # if split.startswith("train"):
        #     nest_dataset = EpochShuffleDataset(
        #         nest_dataset, len(nest_dataset), self.args.seed
        #     )
        # self.datasets[split] = nest_dataset
        if split == "train":
            with data_utils.numpy_seed(self.args.seed):
                shuffle = np.random.permutation(len(src_pocket_dataset_a))

            self.datasets[split] = SortDataset(
                nest_dataset,
                sort_order=[shuffle],
            )
            self.datasets[split] = ResamplingDataset(
                self.datasets[split]
            )
        else:
            self.datasets[split] = nest_dataset


    

    

    

    def build_model(self, args):
        from unicore import models

        model = models.build_model(args, self)
        

        if args.finetune_pocket_model is not None:
            print("load pretrain model weight from...", args.finetune_pocket_model)
            if args.finetune_pocket_model.endswith("pocket_model.pt"):
                import torch
                model.pocket_model.load_state_dict(torch.load(args.finetune_pocket_model, map_location='cpu'), strict=False)
            else:
                state = checkpoint_utils.load_checkpoint_to_cpu(
                    args.finetune_pocket_model,
                )
                model.pocket_model.load_state_dict(state["model"], strict=False)
                

        return model

    def train_step(
        self, sample, model, loss, optimizer, update_num, ignore_grad=True
    ):
        """
        Do forward and backward, and return the loss as computed by *loss*
        for the given *model* and *sample*.

        Args:
            sample (dict): the mini-batch. The format is defined by the
                :class:`~unicore.data.UnicoreDataset`.
            model (~unicore.models.BaseUnicoreModel): the model
            loss (~unicore.losses.UnicoreLoss): the loss
            optimizer (~unicore.optim.UnicoreOptimizer): the optimizer
            update_num (int): the current update
            ignore_grad (bool): multiply loss by 0 if this is set to True

        Returns:
            tuple:
                - the loss
                - the sample size, which is used as the denominator for the
                  gradient
                - logging outputs to display while training
        """


        model.train()
        model.set_num_updates(update_num)
        with torch.autograd.profiler.record_function("forward"):
            loss, sample_size, logging_output = loss(model, sample)
        if ignore_grad:
            loss *= 0
        with torch.autograd.profiler.record_function("backward"):
            optimizer.backward(loss)
        return loss, sample_size, logging_output
    
    def valid_step(self, sample, model, loss, test=False):
        model.eval()
        with torch.no_grad():
            loss, sample_size, logging_output = loss(model, sample)
        return loss, sample_size, logging_output
    

    def load_pockets_dataset(self, data_path):
        """Load a given dataset split.
        'smi','pocket','atoms','coordinates','pocket_atoms','pocket_coordinates','holo_coordinates','holo_pocket_coordinates','scaffold'
        Args:
            split (str): name of the data scoure (e.g., bppp)
        """

 
        dataset = LMDBDataset(data_path)

        dataset = AffinityPocketDataset(
            dataset,
            self.args.seed,
            "pocket_atoms",
            "pocket_coordinates",
            False,
            "pocket"
        )
        poc_dataset = KeyDataset(dataset, "pocket")

        def PrependAndAppend(dataset, pre_token, app_token):
            dataset = PrependTokenDataset(dataset, pre_token)
            return AppendTokenDataset(dataset, app_token)

        dataset = RemoveHydrogenPocketDataset(
            dataset,
            "pocket_atoms",
            "pocket_coordinates",
            "holo_pocket_coordinates",
            True,
            True,
        )
        dataset = CroppingPocketDockingPoseTestDataset(
            dataset,
            self.seed,
            "pocket_atoms",
            "pocket_coordinates",
            "holo_pocket_coordinates",
            self.args.max_pocket_atoms,
        )




        apo_dataset = NormalizeDataset(dataset, "pocket_coordinates")



        src_pocket_dataset = KeyDataset(apo_dataset, "pocket_atoms")
        len_dataset = LengthDataset(src_pocket_dataset)
        src_pocket_dataset = TokenizeDataset(
            src_pocket_dataset,
            self.pocket_dictionary,
            max_seq_len=self.args.max_seq_len,
        )
        coord_pocket_dataset = KeyDataset(apo_dataset, "pocket_coordinates")
        src_pocket_dataset = PrependAndAppend(
            src_pocket_dataset,
            self.pocket_dictionary.bos(),
            self.pocket_dictionary.eos(),
        )
        pocket_edge_type = EdgeTypeDataset(
            src_pocket_dataset, len(self.pocket_dictionary)
        )
        coord_pocket_dataset = FromNumpyDataset(coord_pocket_dataset)
        distance_pocket_dataset = DistanceDataset(coord_pocket_dataset)
        coord_pocket_dataset = PrependAndAppend(coord_pocket_dataset, 0.0, 0.0)
        distance_pocket_dataset = PrependAndAppend2DDataset(
            distance_pocket_dataset, 0.0
        )

        nest_dataset = NestedDictionaryDataset(
            {
                "net_input": {
                    "pocket_src_tokens": RightPadDataset(
                        src_pocket_dataset,
                        pad_idx=self.pocket_dictionary.pad(),
                    ),
                    "pocket_src_distance": RightPadDataset2D(
                        distance_pocket_dataset,
                        pad_idx=0,
                    ),
                    "pocket_src_edge_type": RightPadDataset2D(
                        pocket_edge_type,
                        pad_idx=0,
                    ),
                    "pocket_src_coord": RightPadDatasetCoord(
                        coord_pocket_dataset,
                        pad_idx=0,
                    ),
                },
                "pocket_name": RawArrayDataset(poc_dataset),
            },
        )
        return nest_dataset
    
    